<?php
/**
 * @file
 * Code common to both members.module and members.install files. These are
 * helper functions for the Members Page module.
 */

/**
 * Checks if a url may be added to the menu system. (look for a 404).
 *
 * This function is required for Configurable URL's -- that is, using a Drupal
 * variable in a hook_menu() implementation. When a new url is requested, it
 * must be verified that it is available -- i.e. 404 page not found.
 *
 * @param string $url
 *   The url or Drupal alias requested.  This should be a simple string without
 *   any protocol (http:) prefix and without leading or trailing '/' characters.
 *
 * @returns boolean TRUE | FALSE
 */
function members_check_url_free($url) {
  // Translation function.
  $t = get_t();

  // First check the menu_router table.
  $test = menu_get_item($url);
  if ($test === FALSE) {
    // Looks like there is nothing in the menu_router table.
    // Now check the url_alias table.
    $test = drupal_lookup_path('source', $url);
    if ($test === FALSE) {
      // Nothing in the url_alias table either.
      // One more check is performed if the curl library is installed.
      return function_exists('curl_init') ? _curl_check_url_free($url) : TRUE;
    }
    $msg = $t('The requested url: /') . $url . $t(' is currently an alias for: /') . $test;
    watchdog('members', $msg);
    drupal_set_message($msg);
    return FALSE;
  } else {
    // This path is taken by an menu_router item.
    $msg = 'The requested url: ' . $url . ' is currently used by the "' . $test['title'] . '" module';
    watchdog('members', $msg);
    drupal_set_message($msg);
    return FALSE;
  }
}

/**
 * Checks if a url may be added to the menu system. (looking a 404).
 *
 * This function is required for Configurable URL's -- that is, using a Drupal
 * variable in a hook_menu() implementation. When a new url is requested, it
 * must be verified that it is available -- i.e. 404 page not found.
 *
 * It has been suggested that I should use the Drupal API functions like
 * drupal_lookup_path() or drupal_http_request() or menu_get_item(). I have
 * tried all of these functions. They did not work. All of these functions are
 * much more complicated that the code below. To complicate things,
 * the Drupal API calls differ from one version to the next.
 *
 * It's entirely possible to have a working page on a site that Drupal is
 * blissfully ignorant of -- just create a helloWorld.html -- it works.
 *
 * I therefore humbly submit that this should be a Drupal core function --
 * drupal_check_url_free($url) in support of using Drupal variables in
 * hook_menu() implementations.
 *
 * @param string $url
 *  The url or Drupal alias requested.  This should be a simple string without
 *  any protocol (http:) prefix and without leading or trailing '/' characters.
 *
 * @returns boolean TRUE | FALSE
 */
function _curl_check_url_free($url) {
  global $base_url;

  $url = $base_url . '/' . $url;
  $cptr = curl_init();
  curl_setopt($cptr, CURLOPT_URL, $url);

  // No body required, just the header.
  curl_setopt($cptr, CURLOPT_NOBODY, TRUE);

  // This is a curl request to the local drupal system -- should be a fast response.
  curl_setopt($cptr, CURLOPT_CONNECTTIMEOUT, 2);
  $content = curl_exec($cptr);
  $info = curl_getinfo($cptr);
  $errs = curl_error($cptr);
  if (!empty($errs)) {
    echo "<pre>"; print_r($errs); echo "</pre>";
  }
  curl_close($cptr);

  return $info['http_code'] == '404' ? TRUE : FALSE;
}
